__version__ = "2.0.0"
# __version__ has to be defined in the first line

"""
:mod:`giddy` --- Spatial Dynamics and Mobility
==============================================

"""

from . import directional
from . import ergodic
from . import markov
from . import mobility
from . import rank
from . import util